package sun.nio.cs;

import java.nio.CharBuffer;
import java.nio.charset.CoderResult;

public class Surrogate
{
    public static final char MIN_HIGH = '?';
    public static final char MAX_HIGH = '?';
    public static final char MIN_LOW = '?';
    public static final char MAX_LOW = '?';
    public static final char MIN = '?';
    public static final char MAX = '?';
    public static final int UCS4_MIN = 65536;
    public static final int UCS4_MAX = 1114111;
    
    private Surrogate() {
    }
    
    public static boolean isHigh(int n) {
        return 55296 <= n && n <= 56319;
    }
    
    public static boolean isLow(int n) {
        return 56320 <= n && n <= 57343;
    }
    
    public static boolean is(int n) {
        return 55296 <= n && n <= 57343;
    }
    
    public static boolean neededFor(int codePoint) {
        return Character.isSupplementaryCodePoint(codePoint);
    }
    
    public static char high(int n) {
        assert !(!Character.isSupplementaryCodePoint(n));
        return Character.highSurrogate(n);
    }
    
    public static char low(int n) {
        assert !(!Character.isSupplementaryCodePoint(n));
        return Character.lowSurrogate(n);
    }
    
    public static int toUCS4(char c, char c2) {
        assert Character.isHighSurrogate(c) && Character.isLowSurrogate(c2);
        return Character.toCodePoint(c, c2);
    }
    
    public static class Parser
    {
        private int character;
        private CoderResult error;
        private boolean isPair;
        
        public Parser() {
            this.error = CoderResult.UNDERFLOW;
        }
        
        public int character() {
            assert this.error == null;
            return this.character;
        }
        
        public boolean isPair() {
            assert this.error == null;
            return this.isPair;
        }
        
        public int increment() {
            assert this.error == null;
            return this.isPair ? 2 : 1;
        }
        
        public CoderResult error() {
            assert this.error != null;
            return this.error;
        }
        
        public CoderResult unmappableResult() {
            assert this.error == null;
            return CoderResult.unmappableForLength(this.isPair ? 2 : 1);
        }
        
        public int parse(char c, CharBuffer charBuffer) {
            if (Character.isHighSurrogate(c)) {
                if (!charBuffer.hasRemaining()) {
                    this.error = CoderResult.UNDERFLOW;
                    return -1;
                }
                int value = charBuffer.get();
                if (Character.isLowSurrogate((char)value)) {
                    this.character = Character.toCodePoint(c, (char)value);
                    this.isPair = true;
                    this.error = null;
                    return this.character;
                }
                this.error = CoderResult.malformedForLength(1);
                return -1;
            }
            else {
                if (Character.isLowSurrogate(c)) {
                    this.error = CoderResult.malformedForLength(1);
                    return -1;
                }
                this.character = c;
                this.isPair = false;
                this.error = null;
                return this.character;
            }
        }
        
        public int parse(char c, char[] array, int n, int n2) {
            assert array[n] == c;
            if (Character.isHighSurrogate(c)) {
                if (n2 - n < 2) {
                    this.error = CoderResult.UNDERFLOW;
                    return -1;
                }
                int n3 = array[n + 1];
                if (Character.isLowSurrogate((char)n3)) {
                    this.character = Character.toCodePoint(c, (char)n3);
                    this.isPair = true;
                    this.error = null;
                    return this.character;
                }
                this.error = CoderResult.malformedForLength(1);
                return -1;
            }
            else {
                if (Character.isLowSurrogate(c)) {
                    this.error = CoderResult.malformedForLength(1);
                    return -1;
                }
                this.character = c;
                this.isPair = false;
                this.error = null;
                return this.character;
            }
        }
    }
    
    public static class Generator
    {
        private CoderResult error;
        
        public Generator() {
            this.error = CoderResult.OVERFLOW;
        }
        
        public CoderResult error() {
            assert this.error != null;
            return this.error;
        }
        
        public int generate(int n, int n2, CharBuffer charBuffer) {
            if (Character.isBmpCodePoint(n)) {
                int ch = (char)n;
                if (Character.isSurrogate((char)ch)) {
                    this.error = CoderResult.malformedForLength(n2);
                    return -1;
                }
                if (charBuffer.remaining() < 1) {
                    this.error = CoderResult.OVERFLOW;
                    return -1;
                }
                charBuffer.put((char)ch);
                this.error = null;
                return 1;
            }
            else if (Character.isValidCodePoint(n)) {
                if (charBuffer.remaining() < 2) {
                    this.error = CoderResult.OVERFLOW;
                    return -1;
                }
                charBuffer.put(Character.highSurrogate(n));
                charBuffer.put(Character.lowSurrogate(n));
                this.error = null;
                return 2;
            }
            else {
                this.error = CoderResult.unmappableForLength(n2);
                return -1;
            }
        }
        
        public int generate(int n, int n2, char[] array, int n3, int n4) {
            if (Character.isBmpCodePoint(n)) {
                int ch = (char)n;
                if (Character.isSurrogate((char)ch)) {
                    this.error = CoderResult.malformedForLength(n2);
                    return -1;
                }
                if (n4 - n3 < 1) {
                    this.error = CoderResult.OVERFLOW;
                    return -1;
                }
                array[n3] = ch;
                this.error = null;
                return 1;
            }
            else if (Character.isValidCodePoint(n)) {
                if (n4 - n3 < 2) {
                    this.error = CoderResult.OVERFLOW;
                    return -1;
                }
                array[n3] = Character.highSurrogate(n);
                array[n3 + 1] = Character.lowSurrogate(n);
                this.error = null;
                return 2;
            }
            else {
                this.error = CoderResult.unmappableForLength(n2);
                return -1;
            }
        }
    }
}
